// ---------------------------------------------------------------------------
// pilot_mode.cs -- Version 3.7 -- October 28, 2000
// by Lorne Laliberte (writer@planetstarsiege.com)
//
// Increases your mouse sensitivity in vehicles to make piloting easier
//
// http://www.writerscripts.com
// ---------------------------------------------------------------------------

include("writer\\version.cs");
version("writer\\pilot_mode.cs", "3.7", "Lorne Laliberte", "Oct 28, 2000");

include("writer\\event.cs");
include("writer\\item_events.cs");
include("writer\\jet.cs");
include("writer\\jump.cs");
include("writer\\save.cs");
include("writer\\schedule.cs");
include("writer\\station_events.cs");
include("writer\\weapon_switch.cs");
include("writer\\you_died.cs");

// ---------------------------------------------------------------------------
// DEFAULT SETTINGS:
//
// Edit these if you're running this script for the first time
// All subsequent settings must be done in the config\wp_pilot_mode.cs file

    // Sensitivity you want to use while in pilot mode -- set this to 0 if you
    // don't want your sensitivity changed while in pilot mode.
    $PilotMode::Sensitivity = "0.022";

    // Have pilot_mode.cs try and guess when you've jumped into a scout.
    //
    // The way it does this is by checking to see whether you jumped or used
    // your jets in the last 2 seconds whenever your weapon unmounts.
    //
    // This works very well in most cases, but it can be triggered accidentally
    // if you jump or jet in the 2 seconds before you access an AMMO station.
    //
    // However, if pilot mode is accidentally triggered like this, it will be
    // disabled as soon as you are resupplied by the ammo station.
    //
    // Note: this only happens with ammo stations -- you can access inventory,
    //       vehicle and command stations all day without any problems. :)
    $PilotMode::AutoDetect = "true";

    // Be on the lookout for situations where pilot mode is triggered when you
    // die -- it's most likely a death message that you_died.cs doesn't know
    // about.  If possible, please send me that death message string so I can
    // add support for it in you_died.cs.

    // Disable your jump key while in pilot mode.
    //
    // I use this to prevent myself from accidentally jumping out of vehicles
    // when I instinctively hit my space bar to "jump" up.  If you set this to
    // true, make sure you bind a key to turn off pilot mode (see below) since
    // that will be your new "exit vehicle" key.
    $PilotMode::DisableJumpKey = "false";

    // Turn on pilot mode
    //bind(keyboard0, make, l_shift, "c", to, "PilotMode::On();");

    // Toggle auto-detection of Pilot Mode on/off
    //bind(keyboard0, make, l_shift, "p", TO, "PilotMode::ToggleAutoDetect();");

    // Turn off Pilot Mode and exit the vehicle
    //bind(keyboard0, make, l_shift, "space", to, "PilotMode::Off();");

// ---------------------------------------------------------------------------

WriterPrefs::load("pilot_mode");

// Get the normal mouse sensitivity when Tribes starts up
$Sensitivity::Normal = Client::getMouseSensitivity("playMap.sae");

$PilotMode = 0;

// Toggle auto-detect Pilot Mode on/off
//
function PilotMode::ToggleAutoDetect()
{
    remoteBP(2048, "<L5>Auto Detect Pilot Mode <f2>" @ wif($PilotMode::AutoDetect = !$PilotMode::AutoDetect, "ON", "OFF"),4);

    if($PilotMode::AutoDetect)
        Event::Attach(eventWeaponSwitch, PilotMode::onWeaponSwitch);
    else
        Event::Detach(eventWeaponSwitch, PilotMode::onWeaponSwitch);
}


function PilotMode::onWeaponSwitch(%toWeapon, %fromWeapon)
{
    if(%toWeapon != -1)
    {
        if($PilotMode::youDied)
            $PilotMode::youDied = false;

        if($PilotMode)
            PilotMode::Off(1, 1);

        return;
    }

    if($PilotMode::youDied)
        return;

    // Don't switch unless we jumped in the last 2 seconds
    if( !(wasTapped(pilot_mode_jump) || wasTapped(pilot_mode_jet)) )
        return;

    if($Station::AtStation)
        return;

    if($PilotMode::WeaponDropped)
    {
        $PilotMode::WeaponDropped = false;
        return;
    }

    PilotMode::On();

    return;
}


function PilotMode::onJump()
{
    // Disable jump key while piloting so we don't accidentally jump out
    // of the vehicle...I used to do that all the time and it's a real PITA :)
    if($PilotMode && $PilotMode::DisableJumpKey)
        return mute;

    // Set flag every time we jump that lasts for next 2 seconds
    tap(pilot_mode_jump, 2.0);

    return;
}


function PilotMode::onJetsStarted()
{
    $WithinTapTime[pilot_mode_jet] = true;
    return;
}


function PilotMode::onJetsStopped()
{
    tap(pilot_mode_jet, 2.0);
    return;
}


// Turn Pilot Mode off when we die
//
function PilotMode::onYouDied()
{
    $PilotMode::youDied = true;

    if($PilotMode)
        PilotMode::Off(1, 1);

    return;
}


function PilotMode::AtAmmoStation()
{
    if($PilotMode)
        PilotMode::Off(1, 1);

    return;
}



// Switch to pilot mode, boost mouse sensitivity
// and disable the jump key to avoid accidentally exiting the vehicle
//
function PilotMode::On()
{
    $PilotMode = 1;

    if($PilotMode::Sensitivity)
    {
        setSensitivity($PilotMode::Sensitivity);
        remoteBP(2048, "<jc><f1>Pilot Mode <f2>ON\n<f0>(Mouse Boosted)", 3);
    }
    else
    {
        remoteBP(2048, "<jc><f1>Pilot Mode <f2>ON", 3);
    }

    Event::Trigger(eventPilotModeOn);
}


// Restore normal mouse sensitivity, re-enable the jump key,
// and jump out of the vehicle
//
function PilotMode::Off(%quiet, %nojump)
{
    $PilotMode = 0;

    if($PilotMode::Sensitivity)
        setSensitivity($Sensitivity::Normal);

    if(!%quiet)
        remoteBP(2048, "<jc><f1>Pilot Mode <f2>OFF", 3);

    // Jump out of the vehicle
    if(!%nojump)
        postAction(2048, IDACTION_MOVEUP, 0);

    Event::Trigger(eventPilotModeOff);
}


// Set mouse sensitivity to given value
//
function setSensitivity(%sensitivity)
{
    if(Client::getMouseXaxisFlip("playMap.sae"))
    {
        EditActionMap("playMap.sae");
        bindAction(mouse0, xaxis0, TO, IDACTION_YAW, Scale, %sensitivity);
    }
    else
    {
        EditActionMap("playMap.sae");
        bindAction(mouse0, xaxis0, TO, IDACTION_YAW, Flip, Scale, %sensitivity);
    }

    if(Client::getMouseYaxisFlip("playMap.sae"))
    {
        EditActionMap("playMap.sae");
        bindAction(mouse0, yaxis0, TO, IDACTION_PITCH, Scale, %sensitivity);
    }
    else
    {
        EditActionMap("playMap.sae");
        bindAction(mouse0, yaxis0, TO, IDACTION_PITCH, Flip, Scale, %sensitivity);
    }
}

function pm::jump()
{
    Event::Detach(eventJump, "PilotMode::onJump();");
    Event::Detach(eventJetsStarted, "PilotMode::onJetsStarted();");
    Event::Detach(eventJetsStopped, "PilotMode::onJetsStopped();");
    Event::Attach(eventJump, "PilotMode::onJump();");
}

function pm::jet()
{
    Event::Detach(eventJump, "PilotMode::onJump();");
    Event::Detach(eventJetsStarted, "PilotMode::onJetsStarted();");
    Event::Detach(eventJetsStopped, "PilotMode::onJetsStopped();");
    Event::Attach(eventJetsStarted, "PilotMode::onJetsStarted();");
    Event::Attach(eventJetsStopped, "PilotMode::onJetsStopped();");
}

function pm::both()
{
    Event::Detach(eventJump, "PilotMode::onJump();");
    Event::Detach(eventJetsStarted, "PilotMode::onJetsStarted();");
    Event::Detach(eventJetsStopped, "PilotMode::onJetsStopped();");
    Event::Attach(eventJump, "PilotMode::onJump();");
    Event::Attach(eventJetsStarted, "PilotMode::onJetsStarted();");
    Event::Attach(eventJetsStopped, "PilotMode::onJetsStopped();");
}

function pilot_mode::save()
{
    AddExportText("    // Sensitivity you want to use while in pilot mode -- set this to 0 if you");
    AddExportText("    // don't want your sensitivity changed while in pilot mode.");
    ExportSetting("    $PilotMode::Sensitivity", "0.022");
    AddExportText("");
    AddExportText("    // Have pilot_mode.cs try and guess when you've jumped into a scout.");
    AddExportText("    //");
    AddExportText("    // The way it does this is by checking to see whether you jumped or used");
    AddExportText("    // your jets in the last 2 seconds whenever your weapon unmounts.");
    AddExportText("    //");
    AddExportText("    // This works very well in most cases, but it can be triggered accidentally");
    AddExportText("    // if you jump or jet in the 2 seconds before you access an AMMO station.");
    AddExportText("    //");
    AddExportText("    // However, if pilot mode is accidentally triggered like this, it will be");
    AddExportText("    // disabled as soon as you are resupplied by the ammo station.");
    AddExportText("    //");
    AddExportText("    // Note: this only happens with ammo stations -- you can access inventory,");
    AddExportText("    //       vehicle and command stations all day without any problems. :)");
    ExportSetting("    $PilotMode::AutoDetect", true);
    AddExportText("");
    AddExportText("    // Be on the lookout for situations where pilot mode is triggered when you");
    AddExportText("    // die -- it's most likely a death message that you_died.cs doesn't know");
    AddExportText("    // about.  If possible, please send me that death message string so I can");
    AddExportText("    // add support for it in you_died.cs");
    AddExportText("");
    AddExportText("    // Disable your jump key while in pilot mode.");
    AddExportText("    //");
    AddExportText("    // I use this to prevent myself from accidentally jumping out of vehicles");
    AddExportText("    // when I instinctively hit my space bar to \"jump\" up.  If you set this to");
    AddExportText("    // true, make sure you bind a key to turn off pilot mode (see below) since");
    AddExportText("    // that will be your new \"exit vehicle\" key.");
    ExportSetting("    $PilotMode::DisableJumpKey", false);
    AddExportText("");
    AddExportText("    // Turn on pilot mode");

    ExportBind("PilotMode::On();", "//bind(keyboard0, make, l_shift, \"c\", TO, \"PilotMode::On();\");");

    AddExportText("");
    AddExportText("    // Toggle auto-detection of Pilot Mode on/off");

    ExportBind("PilotMode::ToggleAutoDetect();", "//bind(keyboard0, make, l_shift, \"p\", TO, \"PilotMode::ToggleAutoDetect();\");");

    AddExportText("");
    AddExportText("    // Turn off Pilot Mode and exit the vehicle");

    ExportBind("PilotMode::Off();", "//bind(keyboard0, make, l_shift, \"space\", TO, \"PilotMode::Off();\");");
}

WriterPrefs::save("pilot_mode");

function pilot_mode::init()
{
    needs("writer\\pilot_mode.cs", "writer\\event.cs 2.7",
                                   "writer\\item_events.cs 2.12",
                                   "writer\\jet.cs 1.4",
                                   "writer\\jump.cs 2.7",
                                   "writer\\save.cs 2.3",
                                   "writer\\schedule.cs 1.3",
                                   "writer\\station_events.cs 4.8",
                                   "writer\\weapon_switch.cs 2.5",
                                   "writer\\you_died.cs 1.6");

    Event::Attach(eventDropWeapon, "$PilotMode::WeaponDropped=1;");
    Event::Attach(eventWeaponSwitch, PilotMode::onWeaponSwitch);
    Event::Attach(eventResupplyComplete, "PilotMode::Off(1,1);");
    Event::Attach(eventJump, "PilotMode::onJump();");
    Event::Attach(eventJetsStarted, "PilotMode::onJetsStarted();");
    Event::Attach(eventJetsStopped, "PilotMode::onJetsStopped();");
    Event::Attach(eventYouDied, "PilotMode::onYouDied();");
    Event::Attach(eventChangeMission, "PilotMode::Off(1,1); $PilotMode::YouDied=1;");
    Event::Attach(eventExit, "setSensitivity($Sensitivity::Normal);");
    Event::Attach(eventAtAmmoStation, "PilotMode::AtAmmoStation();");
}
Event::Attach(writerInit, "pilot_mode::init();");

if(!$UsingWriterLoader) pilot_mode::init();
